@@ -5,7 +5,7 @@
  * NO WARRANTY, not even implied warranties. Contains trade secrets.
  * Distribution prohibited unless authorized in writing.
  * Licensed under Apache License 2.0, see file COPYING.
- * $Id: zxidpool.c,v 1.4 2009-10-18 12:39:10 sampo Exp $
+ * $Id: zxidpool.c,v 1.7 2009-11-24 23:53:40 sampo Exp $
  *
  * 4.9.2009, forked from zxidsimp.c --Sampo
  */
@@ -48,7 +48,7 @@ struct zx_str* zxid_pool_to_ldif(struct zxid_conf* cf, struct zxid_attr* pool)
 	D("attribute(%s) filtered out by del rule in OUTMAP", at->name);
 	continue;
       }
-      at->map_val = zxid_map_val(cf, map, zx_ref_str(cf->ctx, at->val));
+      at->map_val = zxid_map_val(cf, map, zx_ref_str(cf->ctx, STRNULLCHK(at->val)));
       if (map->dst && map->dst[0] && map->src && map->src[0] != '*') {
 	name_len = strlen(map->dst);
       } else {
@@ -58,7 +58,7 @@ struct zx_str* zxid_pool_to_ldif(struct zxid_conf* cf, struct zxid_attr* pool)
       DD("len1=%d", len);
 
       for (av = at->nv; av; av = av->n) {
-	av->map_val = zxid_map_val(cf, map, zx_ref_str(cf->ctx, av->val));
+	av->map_val = zxid_map_val(cf, map, zx_ref_str(cf->ctx, STRNULLCHK(av->val)));
 	len += name_len + sizeof(": \n")-1 + av->map_val->len;
 	DD("len2=%d", len);
       }
@@ -140,8 +140,10 @@ struct zx_str* zxid_pool_to_ldif(struct zxid_conf* cf, struct zxid_attr* pool)
       p += name_len;
       *p++ = ':';
       *p++ = ' ';
-      strcpy(p, at->val);
-      p += strlen(at->val);
+      if (at->val) {
+	strcpy(p, at->val);
+	p += strlen(at->val);
+      }
       *p++ = '\n';
 
       DD("len 3=%d name_len=%d name(%s)", p-ss->s, name_len, at->name);
@@ -151,8 +153,10 @@ struct zx_str* zxid_pool_to_ldif(struct zxid_conf* cf, struct zxid_attr* pool)
 	p += name_len;
 	*p++ = ':';
 	*p++ = ' ';
-	strcpy(p, av->val);
-	p += strlen(av->val);
+	if (at->val) {
+	  strcpy(p, av->val);
+	  p += strlen(av->val);
+	}
 	*p++ = '\n';
 
 	D("len 4=%d", p-ss->s);
@@ -169,7 +173,7 @@ struct zx_str* zxid_pool_to_ldif(struct zxid_conf* cf, struct zxid_attr* pool)
 /*() Convert attributes from (session) pool to JSON, applying OUTMAP.
  * *** Need to check escaping JSON values, e.g. " or \n */
 
-/* Called by: */
+/* Called by:  zxid_simple_ab_pep */
 struct zx_str* zxid_pool_to_json(struct zxid_conf* cf, struct zxid_attr* pool)
 {
   char* p;
@@ -189,7 +193,7 @@ struct zx_str* zxid_pool_to_json(struct zxid_conf* cf, struct zxid_attr* pool)
 	D("attribute(%s) filtered out by del rule in OUTMAP", at->name);
 	continue;
       }
-      at->map_val = zxid_map_val(cf, map, zx_ref_str(cf->ctx, at->val));
+      at->map_val = zxid_map_val(cf, map, zx_ref_str(cf->ctx, STRNULLCHK(at->val)));
       if (map->dst && map->dst[0] && map->src && map->src[0] != '*') {
 	name_len = strlen(map->dst);
       } else {
@@ -199,7 +203,7 @@ struct zx_str* zxid_pool_to_json(struct zxid_conf* cf, struct zxid_attr* pool)
       if (at->nv) {  /* Multivalue requires array */
 	len += name_len + sizeof("\"\":[\"\"],")-1 + at->map_val->len;
 	for (av = at->nv; av; av = av->n) {
-	  av->map_val = zxid_map_val(cf, map, zx_ref_str(cf->ctx, av->val));
+	  av->map_val = zxid_map_val(cf, map, zx_ref_str(cf->ctx, STRNULLCHK(av->val)));
 	  len += name_len + sizeof(",\"\"")-1 + av->map_val->len;
 	}
       } else {
@@ -269,21 +273,27 @@ struct zx_str* zxid_pool_to_json(struct zxid_conf* cf, struct zxid_attr* pool)
       if (at->nv) {
 	*p++ = '[';
 	*p++ = '"';
-	strcpy(p, at->val);
-	p += strlen(at->val);
+	if (at->val) {
+	  strcpy(p, at->val);
+	  p += strlen(at->val);
+	}
 	*p++ = '"';
 	for (av = at->nv; av; av = av->n) {
 	  *p++ = ',';
 	  *p++ = '"';
-	  strcpy(p, av->val);
-	  p += strlen(av->val);
+	  if (at->val) {
+	    strcpy(p, av->val);
+	    p += strlen(av->val);
+	  }
 	  *p++ = '"';
 	}
 	*p++ = ']';
       } else {
 	*p++ = '"';
-	strcpy(p, at->val);
-	p += strlen(at->val);
+	if (at->val) {
+	  strcpy(p, at->val);
+	  p += strlen(at->val);
+	}
 	*p++ = '"';
       }
     }
@@ -300,7 +310,7 @@ struct zx_str* zxid_pool_to_json(struct zxid_conf* cf, struct zxid_attr* pool)
  * *** Need to figure out how to distinguish query string return from
  *     other returns, like redirect. Perhaps arrange dn field always first? */
 
-/* Called by: */
+/* Called by:  zxid_simple_ab_pep */
 struct zx_str* zxid_pool_to_qs(struct zxid_conf* cf, struct zxid_attr* pool)
 {
   char* p;
@@ -320,7 +330,7 @@ struct zx_str* zxid_pool_to_qs(struct zxid_conf* cf, struct zxid_attr* pool)
 	D("attribute(%s) filtered out by del rule in OUTMAP", at->name);
 	continue;
       }
-      at->map_val = zxid_map_val(cf, map, zx_ref_str(cf->ctx, at->val));
+      at->map_val = zxid_map_val(cf, map, zx_ref_str(cf->ctx, STRNULLCHK(at->val)));
       if (map->dst && map->dst[0] && map->src && map->src[0] != '*') {
 	name_len = strlen(map->dst);
       } else {
@@ -328,21 +338,23 @@ struct zx_str* zxid_pool_to_qs(struct zxid_conf* cf, struct zxid_attr* pool)
       }
       len += name_len + sizeof("=&")-1 + zx_url_encode_len(at->map_val->len,at->map_val->s)-1;
       for (av = at->nv; av; av = av->n) {
-	av->map_val = zxid_map_val(cf, map, zx_ref_str(cf->ctx, av->val));
+	av->map_val = zxid_map_val(cf, map, zx_ref_str(cf->ctx, STRNULLCHK(av->val)));
 	len += name_len + sizeof("=&")-1 + zx_url_encode_len(av->map_val->len,av->map_val->s)-1;
       }
       D("len=%d name_len=%d %s", len, name_len, at->name);
     } else {
       name_len = strlen(at->name);
-      len += name_len + sizeof("=&")-1 + (at->val?zx_url_encode_len(strlen(at->val),at->val):0)-1;
-      D("len=%d name_len=%d %s (nomap) url_enc_len=%d", len, name_len, at->name, (at->val?zx_url_encode_len(strlen(at->val),at->val):0));
+      len += name_len + sizeof("=&")-1 + (at->val?zx_url_encode_len(strlen(at->val),at->val)-1:0);
+      D("len=%d name_len=%d %s (nomap) url_enc_len=%d", len, name_len, at->name, (at->val?zx_url_encode_len(strlen(at->val),at->val)-1:0));
       for (av = at->nv; av; av = av->n)
-	len += name_len + sizeof("=&")-1 + (av->val?zx_url_encode_len(strlen(av->val),av->val):0)-1;
+	len += name_len + sizeof("=&")-1 + (av->val?zx_url_encode_len(strlen(av->val),av->val)-1:0);
     }
   }
   
   /* Attribute rendering pass */
 
+  DD("HERE %d", 0);
+
   ss = zx_new_len_str(cf->ctx, len);
   p = ss->s;
   memcpy(p, "dn=QS1&", sizeof("dn=QS1&")-1);
@@ -378,14 +390,16 @@ struct zx_str* zxid_pool_to_qs(struct zxid_conf* cf, struct zxid_attr* pool)
       strcpy(p, at->name);
       p += name_len;
       *p++ = '=';
-      p = zx_url_encode_raw(strlen(at->val), at->val, p);
+      if (at->val)
+	p = zx_url_encode_raw(strlen(at->val), at->val, p);
       *p++ = '&';
       
       for (av = at->nv; av; av = av->n) {
 	strcpy(p, at->name);
 	p += name_len;
 	*p++ = '=';
-	p = zx_url_encode_raw(strlen(av->val), av->val, p);
+	if (at->val)
+	  p = zx_url_encode_raw(strlen(av->val), av->val, p);
 	*p++ = '&';
       }
     }
@@ -399,6 +413,7 @@ struct zx_str* zxid_pool_to_qs(struct zxid_conf* cf, struct zxid_attr* pool)
 
 /*() Add values, applying NEED, WANT, and INMAP */
 
+/* Called by:  zxid_add_a7n_at_to_pool x2 */
 static int zxid_add_at_values(struct zxid_conf* cf, struct zxid_ses* ses, struct zx_sa_Attribute_s* at, char* name, struct zx_str* issuer)
 {
   struct zx_str* ss;
@@ -426,10 +441,10 @@ static int zxid_add_at_values(struct zxid_conf* cf, struct zxid_ses* ses, struct
   ses->at->orig = at;
   ses->at->issuer = issuer;
   
-  for (av = at->AttributeValue; av; av = ZX_NEXT(av)) {
+  for (av = at->AttributeValue; av; av = (struct zx_sa_AttributeValue_s*)ZX_NEXT(av)) {
     D("Adding value: %p", av->gg.content);
     if (av->EndpointReference || av->ResourceOffering)
-      continue;  /* Skip bootstraps. They are handled elsewhere. */
+      continue;  /* Skip bootstraps. They are handled elsewhere, see zxid_snarf_eprs_from_ses(). */
     if (av->gg.content) {
       ss = zxid_map_val(cf, map, av->gg.content);
       if (ses->at->val) {
@@ -447,6 +462,7 @@ static int zxid_add_at_values(struct zxid_conf* cf, struct zxid_ses* ses, struct
 
 /*() Add Attribute Statements of an Assertion to pool, applying NEED, WANT, and INMAP */
 
+/* Called by:  zxid_ses_to_pool */
 static void zxid_add_a7n_at_to_pool(struct zxid_conf* cf, struct zxid_ses* ses, struct zx_sa_Assertion_s* a7n)
 {
   struct zx_sa_Attribute_s* at;
@@ -454,8 +470,8 @@ static void zxid_add_a7n_at_to_pool(struct zxid_conf* cf, struct zxid_ses* ses,
   if (!a7n)
     return;
   
-  for (as = a7n->AttributeStatement; as; as = ZX_NEXT(as)) {
-    for (at = as->Attribute; at; at = ZX_NEXT(at)) {      
+  for (as = a7n->AttributeStatement; as; as = (struct zx_sa_AttributeStatement_s*)ZX_NEXT(as)) {
+    for (at = as->Attribute; at; at = (struct zx_sa_Attribute_s*)ZX_NEXT(at)) {      
       if (at->Name)
 	zxid_add_at_values(cf, ses, at, zx_str_to_c(cf->ctx, at->Name), a7n->Issuer ? a7n->Issuer->gg.content : 0);
       if (at->FriendlyName)
@@ -466,6 +482,7 @@ static void zxid_add_a7n_at_to_pool(struct zxid_conf* cf, struct zxid_ses* ses,
 
 /*() Add simple attribute to pool, applying NEED, WANT, and INMAP */
 
+/* Called by:  zxid_add_qs_to_pool, zxid_ses_to_pool x8, zxid_simple_ab_pep x2 */
 void zxid_add_attr_to_pool(struct zxid_conf* cf, struct zxid_ses* ses, char* at_name, struct zx_str* val)
 {
   struct zxid_map* map;
@@ -493,6 +510,7 @@ void zxid_add_attr_to_pool(struct zxid_conf* cf, struct zxid_ses* ses, char* at_
  * are applied. The pool is suitable for use by PEP or eventually
  * rendering to LDIF (or JSON). */
 
+/* Called by:  zxid_simple_ab_pep */
 void zxid_ses_to_pool(struct zxid_conf* cf, struct zxid_ses* ses)
 {
   char* src;
@@ -500,6 +518,7 @@ void zxid_ses_to_pool(struct zxid_conf* cf, struct zxid_ses* ses)
   char* lim;
   struct zx_str* issuer;
   struct zx_str* accr;
+  D_INDENT("ses_to_pool: ");
   zxid_get_ses_sso_a7n(cf, ses);
   D("adding a7n %p to pool", ses->a7n);
   zxid_add_a7n_at_to_pool(cf, ses, ses->a7n);
@@ -512,29 +531,36 @@ void zxid_ses_to_pool(struct zxid_conf* cf, struct zxid_ses* ses)
   accr = ses->a7n&&ses->a7n->AuthnStatement&&ses->a7n->AuthnStatement->AuthnContext&&ses->a7n->AuthnStatement->AuthnContext->AuthnContextClassRef&&ses->a7n->AuthnStatement->AuthnContext->AuthnContextClassRef->content&&ses->a7n->AuthnStatement->AuthnContext->AuthnContextClassRef->content?ses->a7n->AuthnStatement->AuthnContext->AuthnContextClassRef->content:0;
   zxid_add_attr_to_pool(cf, ses, "authnctxlevel", accr);
 
-  zxid_add_attr_to_pool(cf, ses, "idpnid",     zx_dup_str(cf->ctx, STRNULLCHKD(ses->nid)));
+  zxid_add_attr_to_pool(cf, ses, "idpnid",     zx_dup_str(cf->ctx, STRNULLCHK(ses->nid)));
+  zxid_add_attr_to_pool(cf, ses, "tgtnid",     zx_dup_str(cf->ctx, STRNULLCHK(ses->tgt)));
   zxid_add_attr_to_pool(cf, ses, "eid",        zxid_my_entity_id(cf));
-  zxid_add_attr_to_pool(cf, ses, "sesid",      zx_dup_str(cf->ctx, STRNULLCHKD(ses->sid)));
-  zxid_add_attr_to_pool(cf, ses, "setcookie",  zx_dup_str(cf->ctx, STRNULLCHKD(ses->setcookie)));
-  zxid_add_attr_to_pool(cf, ses, "cookie",     zx_dup_str(cf->ctx, STRNULLCHKD(ses->cookie)));
-  zxid_add_attr_to_pool(cf, ses, "rs",         zx_dup_str(cf->ctx, STRNULLCHKD(ses->rs)));
+  zxid_add_attr_to_pool(cf, ses, "sesid",      zx_dup_str(cf->ctx, STRNULLCHK(ses->sid)));
+  zxid_add_attr_to_pool(cf, ses, "setcookie",  zx_dup_str(cf->ctx, STRNULLCHK(ses->setcookie)));
+  zxid_add_attr_to_pool(cf, ses, "cookie",     zx_dup_str(cf->ctx, STRNULLCHK(ses->cookie)));
+  zxid_add_attr_to_pool(cf, ses, "ssoa7npath", zx_dup_str(cf->ctx, STRNULLCHK(ses->sso_a7n_path)));
+  zxid_add_attr_to_pool(cf, ses, "tgta7npath", zx_dup_str(cf->ctx, STRNULLCHK(ses->tgt_a7n_path)));
+  zxid_add_attr_to_pool(cf, ses, "msgid",      zx_dup_str(cf->ctx, STRNULLCHK(ses->msgid)));
+  zxid_add_attr_to_pool(cf, ses, "rs",         zx_dup_str(cf->ctx, STRNULLCHK(ses->rs)));
 
   src = dst = ses->at->val;
   lim = ses->at->val + strlen(ses->at->val);
   URL_DECODE(dst, src, lim);
   *dst = 0;
   D("RelayState(%s)", ses->at->val);
+  D_DEDENT("ses_to_pool: ");
 }
 
-
 /*(i) Add Attributes from Querty String to Session attribute pool
  * The qs argument is parsed according to the CGI Query String rules
  * and the attributes are added to the session. If apply_map is 1, the
  * INMAP configuration is applied. While this may seem a hassle, it
  * allows for specification of the values as safe_base64, etc. If values
  * are to be added verbatim, just specify 0 (all other values reserved).
+ * The input argument qs gets modified in-situ due to URL decoding and
+ * nul termination. Make sure to duplicate any string constant before calling.
  * Returns 1 on success, 0 on failure (return value often not checked). */
 
+/* Called by:  zxid_az_cf_ses */
 int zxid_add_qs_to_pool(struct zxid_conf* cf, struct zxid_ses* ses, char* qs, int apply_map)
 {
   char *p, *n, *v, *val, *name;
@@ -584,6 +610,7 @@ int zxid_add_qs_to_pool(struct zxid_conf* cf, struct zxid_ses* ses, char* qs, in
  * attribute name is in the internal namespace). Other apply_map values
  * are reserved. */
 
+/* Called by: */
 struct zx_str* zxid_get_at(struct zxid_conf* cf, struct zxid_ses* ses, char* atname, int ix, int apply_map)
 {
   struct zxid_attr* at;
